@@ -60,7 +60,12 @@ def download_dataset(dataset_name: str, project_id: str = None, version: str = N
         project_id = project.get('id')
 
     manager = load_event_manager()
-
+    if version is not None:
+        dirpath = os.path.join(save_path, dataset_name+'-'+version)
+        if os.path.isdir(dirpath) and overwrite==False:
+            logger.error(f'Dataset is already downloaded at {dirpath}. Delete this folder or set overwrite==True to download again.')
+            return dirpath
+            
     # Fetch the download info
     res = manager.add_event('get_dataset_download_info', 
             args=(dataset_name, project_id, version), 
