@@ -61,6 +61,12 @@ def download_mlmodel(mlmodel_name: str, project_id: str = None, version: str = N
 
     manager = load_event_manager()
 
+    if version is not None:
+        dirpath = os.path.join(save_path, mlmodel_name+'-'+version)
+        if os.path.isdir(dirpath) and overwrite==False:
+            logger.error(f'Model is already downloaded at {dirpath}. Delete this folder or set overwrite==True to download again.')
+            return dirpath
+
     # Fetch the download info
     res = manager.add_event('get_mlmodel_download_info', 
             args=(mlmodel_name, project_id, version), 
