@@ -6,6 +6,7 @@ import threading
 import logging
 import time
 from typing import List
+from more_itertools import chunked
 
 logger = logging.getLogger('hectiqlab')
 
@@ -148,6 +149,13 @@ def set_note(run_id, note, token):
                      body_args=dict(text=note))
     return res
 
+def set_paper(run_id, content, token):
+    res = lab_request(path=f"/runs/{run_id}/paper",
+                     request_type="POST",
+                     token=token,
+                     body_args=dict(source=content))
+    return res
+
 def add_tag(run_id, name, description, color, token):
     res = lab_request(path=f"/runs/{run_id}/tags",
                      request_type="POST",
@@ -159,7 +167,14 @@ def log_mlmodel(run_id, mlmodel_name, version, project_id, token):
     res = lab_request(path=f"/runs/{run_id}/log_mlmodel",
                      request_type="POST",
                      token=token,
-                     query_args=dict(mlmodel_name=mlmodel_name, version=version, project_id=12))
+                     query_args=dict(mlmodel_name=mlmodel_name, version=version, project_id=project_id))
+    return res
+
+def log_dataset(run_id, dataset_name, version, project_id, token):
+    res = lab_request(path=f"/runs/{run_id}/log_dataset",
+                     request_type="POST",
+                     token=token,
+                     query_args=dict(dataset_name=dataset_name, version=version, project_id=project_id))
     return res
 
 def append_logs(run_id, logs, token):
@@ -228,6 +243,7 @@ def add_mlmodel(run_id, filenames: List[str], full_paths: List[str], num_bytes:
         print(res.get('detail'))
         return
 
+    mlmodel = res.get('mlmodel')
     for filename, content in res['policies'].items():
         print(f'Uploading {filename}')
         if filename is None:
@@ -247,44 +263,60 @@ def add_mlmodel(run_id, filenames: List[str], full_paths: List[str], num_bytes:
         lab_request(path=route,
                      request_type="GET",
                      token=token)
-    return res
+    return mlmodel
 
 def add_dataset(run_id, filenames: List[str], full_paths: List[str], num_bytes: List[int], name: str, short_description: str, version: str, token: str = None):
 
-    # Get policy
-    body = dict(run_id=run_id, name=name, filenames=filenames, 
-        short_description=short_description, 
-        num_bytes=num_bytes, 
-        version=version)
-
+    # Create dataset
+    body = dict(run_id=run_id, name=name, filenames=[], 
+            short_description=short_description, 
+            num_bytes=[], 
+            version=version)
     res = lab_request(path=f"/runs/{run_id}/dataset",
-                     request_type="POST",
-                     token=token,
-                     body_args=body)
-    if res.get('status_code') == 400:
-        print(res.get('detail'))
-        return
+                         request_type="POST",
+                         token=token,
+                         body_args=body)
+    assert res.get('status_code') == 200, f"An error occured while creating the dataset: {res.get('detail')}"
+    dataset = res.get('dataset')
+    print(f"Creating dataset {dataset.get('name')}, version {dataset.get('version')}.")
 
-    for filename, content in res['policies'].items():
-        print(f'Uploading {filename}')
-        if filename is None:
-            continue
-        content_bytes = open(full_paths[filename], 'rb')
+    batch_size = 20
+    for batch in chunked(zip(filenames, full_paths, num_bytes), batch_size):
+        batch_filenames = list(zip(*batch))[0]
+        batch_full_paths = list(zip(*batch))[1]
+        batch_num_bytes = list(zip(*batch))[2]
 
-        logger.debug(filename, content)
-        policy, bucket_name, route = content["policy"], content["bucket_name"], content['route']
+        # Get policy
+        body = dict(run_id=run_id, filenames=batch_filenames, 
+            num_bytes=batch_num_bytes)
 
-        # Send file
-        file = {"file": (bucket_name, content_bytes)}
+        res = lab_request(path=f"/runs/{run_id}/add-files-to-dataset",
+                         request_type="POST",
+                         token=token,
+                         query_args=dict(dataset_uuid=dataset.get('uuid')),
+                         body_args=body)
+        assert res.get('status_code') == 200, f"An error occured while pushing new files: {res.get('detail')}"
 
-        logger.debug(policy["url"], policy["fields"])
-        requests.post(policy["url"], data=policy["fields"], files=file)
+        for filename, content in res['policies'].items():
+            print(f'Uploading {filename}')
+            if filename is None:
+                continue
+            content_bytes = open(full_paths[filename], 'rb')
 
-        # Callback
-        lab_request(path=route,
-                     request_type="GET",
-                     token=token)
-    return res
+            logger.debug(filename, content)
+            policy, bucket_name, route = content["policy"], content["bucket_name"], content['route']
+
+            # Send file
+            file = {"file": (bucket_name, content_bytes)}
+
+            logger.debug(policy["url"], policy["fields"])
+            requests.post(policy["url"], data=policy["fields"], files=file)
+
+            # Callback
+            lab_request(path=route,
+                         request_type="GET",
+                         token=token)
+    return dataset
 
 
 def add_shared_artifact(project_id: str, filename: str, content_bytes: bytes, num_bytes: int = None, token: str = None):
