@@ -17,6 +17,7 @@ from pyhectiqlab.mlmodels import upload_mlmodel as ops_upload_mlmodel
 from pyhectiqlab.datasets import download_dataset as ops_download_dataset
 from pyhectiqlab.datasets import upload_dataset as ops_upload_dataset
 from pyhectiqlab.mlmodels import mlmodel_info_at_path
+from pyhectiqlab.datasets import dataset_info_at_path
 
 from functools import partial, wraps
 from typing import Optional
@@ -150,7 +151,7 @@ class Run():
         self._read_only = True
 
     @write_method
-    def add_artifacts(self, filepath: str, step: int = None):
+    def add_artifacts(self, filepath: str, step: int = None, wait_response: bool = False):
         """Log a file as an artifacts. If a file already exists with the name, it will be overwritten.
 
         filepath [str] : Path to the file.
@@ -160,10 +161,9 @@ class Run():
         num_bytes = os.path.getsize(filepath)
         content_bytes = open(filepath, "rb")
         args = (self._id, filename, content_bytes, num_bytes, step)
-        self.events_manager.add_event("add_file", args)
+        return self.events_manager.add_event("add_file", args, async_method=bool(1-wait_response))
 
     @write_method
-    @beta
     def add_mlmodel_usage(self, mlmodel_name: str, version: str = None):
         """
         """
@@ -171,7 +171,6 @@ class Run():
         self.events_manager.add_event("log_mlmodel", args)
 
     @write_method
-    @beta
     def add_mlmodel_usage_from_dirpath(self, dirpath:str):
         """Log that this run is using a certain model. 
         """
@@ -196,12 +195,14 @@ class Run():
         short_description: Short description.
         push_dir: Must be set to True if the source_path is a directory
         """
-        ops_upload_mlmodel(source_path=source_path, 
+        mlmodel = ops_upload_mlmodel(source_path=source_path, 
             mlmodel_name=name,
              run_id=self._id, 
              version=version,
              short_description=short_description,
              push_dir=push_dir)
+        if mlmodel is not None:
+            self.add_mlmodel_usage(mlmodel.get('name'), version=mlmodel.get('version'))
 
     def download_mlmodel(self, mlmodel_name: str, version: str = None, save_path: str = './', overwrite:bool = False):
         """Download an existing mlmodel from the run's project.
@@ -218,7 +219,7 @@ class Run():
                         save_path=save_path, 
                         overwrite=overwrite)
         if dirpath is not None:
-            print(dirpath) 
+            self.add_mlmodel_usage_from_dirpath(dirpath)
         return dirpath
         
     @write_method
@@ -234,12 +235,33 @@ class Run():
         short_description: Short description.
         push_dir: Must be set to True if the source_path is a directory
         """
-        ops_upload_dataset(source_path=source_path, 
+        dataset = ops_upload_dataset(source_path=source_path, 
             dataset_name=name,
              run_id=self._id, 
              version=version,
              short_description=short_description,
              push_dir=push_dir)
+        if dataset is not None:
+            self.add_dataset_usage(dataset.get('name'), version=dataset.get('version'))
+
+    @write_method
+    def add_dataset_usage(self, dataset_name: str, version: str = None):
+        """
+        """
+        args = (self._id, dataset_name, version, self.project_view['id'])
+        self.events_manager.add_event("log_dataset", args)
+
+    @write_method
+    def add_dataset_usage_from_dirpath(self, dirpath:str):
+        """Log that this run is using a certain dataset. 
+        """
+        meta = dataset_info_at_path(dirpath)
+        assert meta is not None, f'Could not find the dataset at {dirpath}'
+
+        print(f"{meta.get('name')} - {meta.get('version')}")
+        args = (self._id, meta.get('name'), meta.get('version'), self.project_view['id'])
+        self.events_manager.add_event("log_dataset", args)
+        return
 
     def download_dataset(self, dataset_name: str, version: str = None, save_path: str = './', overwrite:bool = False):
         """Download an existing dataset from the run's project.
@@ -256,7 +278,7 @@ class Run():
                         save_path=save_path, 
                         overwrite=overwrite)
         if dirpath is not None:
-            print(dirpath)        
+            self.add_dataset_usage_from_dirpath(dirpath)   
         return dirpath
     
     @write_method
@@ -422,12 +444,19 @@ class Run():
         self.events_manager.add_event("push_meta", args)
         
     @write_method
-    def set_note(self, text):
+    def set_note(self, text: str):
         """Add or update a custom note field for the run.
         """
         args = (self._id, text)
         self.events_manager.add_event("set_note", args)
-        
+    
+    @write_method
+    def set_paper(self, content: str):
+        """Overwrite the content of the paper of the run.
+        """
+        args = (self._id, content)
+        self.events_manager.add_event("set_paper", args)
+
     @write_method
     def start_recording_logs(self, logger_name=None):
         """Start tailing the logs on a logger name.
